//e_cam33_app.c --- e_cam33 control application source file


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <unistd.h>
#include <assert.h>
#include <fcntl.h>      
#include <errno.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <linux/kernel.h>
//#include <linux/videodev.h>
#include <linux/videodev2.h>


#include "e_cam33_app.h"
#include <e_cam33_api.h>


//Declaration of global variables
FILE *lockfile = NULL;

int main(int argc, char *argv[])
{
	int ret = 0;
	int main_option = 0;	
	INT32 SensorID, AFStatus;
	INT16 curr_manual_position;
	char scan_string[8];
	
	//Find whether another instance of control application is running	
	if(!find_lock_file())
	{
		return 1;
	}

	// Initialize the e_cam33 Library
	ret=InitLibrary();

	if(ret == FAILURE)
	{
		printf("InitLibrary is Failed \n");
		goto end_of_app;
	}
				
	// IMPORTANT NOTE: To do any UVC XU Control, the video streaming should be started first(by using any application for example guvcview, cheese)
		
	while(1)
	{
		printf("-----------------------------\n");
		printf("          Main Menu          \n");
		printf("-----------------------------\n");
		printf("[1] READ SENSOR ID\n[2] APPLY SINGLE TRIGGER FOCUS \n[3] READ AF STATUS \n[4] EXIT FROM THE APPLICATION \n");
		printf("Enter the option :\n");
		scanf("%s",scan_string);
		main_option = strtoul(scan_string,NULL,10);
		switch(main_option)
		{
			case READ_SENSORID:
			{
				ret = GetSensorId(&SensorID);
				if(ret == FAILURE)
				{
					printf("Failed to Get Sensor ID \n");
				}
				printf("SENSOR ID :0X%X\n",SensorID);
			}	
			break;
			case SINGLE_TRIGGER:
			{
				if((AFFreezeControl(SINGLE_TRIG_AF)) != SINGLE_TRIG_AF)
				{
					printf("Single Trigger Auto Focus Fails \n");
				}
			}
			break;
			case READ_AF_STATUS:
			{
				if(GetAFStatus(&AFStatus) == SUCCESS)
				{	
					switch(AFStatus)
					{
						case AF_CONT_FROZEN:
						{
							printf("Sensor is in CONTINUOUS AUTO FOCUS - FROZEN\n");
						}
						break;
						case AF_CONT_FOCUSED:
						{
							printf("Sensor is in CONTINUOUS AUTO FOCUS - FOCUSED\n");
						}
						break;
						case MANUAL_FOCUS:
						{
							printf("Sensor is in MANUAL FOCUS\n");
						}
						break;
						case AF_CONT_FOCUSING:
						{
							printf("Sensor is in CONTINUOUS AUTO FOCUS - FOCUSING\n");
						}
						break;
						case AF_SINGLE_TRIGGER_FOCUSING:
						{
							printf("AF Status is: SINGLE TRIGGER AUTO FOCUS - FOCUSING\n");
						}
						break;
						case AF_SINGLE_TRIGGER_FOCUSED:
						{
							printf("Sensor is in SINGLE TRIGGER AUTO FOCUS - FOCUSED\n");
						}
						break;
						case AF_SINGLE_TRIGGER_FROZEN:
						{
							printf("Sensor is in SINGLE TRIGGER AUTO FOCUS - FROZEN\n");
						}
						break;
					}	
				}
				else
					printf("Error in reading AF Status\n");
			}	
			break;
			case EXIT_FROM_APP:
			{
				printf("FUNC %s(): LINE %d: Exiting from Application \n",__FUNCTION__,__LINE__);
				goto end_of_app;
			}
			break;
			default:	
			{
				printf("Enter the right option \n");
			}
			break;
		} // end of switch loop
	
	} //end of while loop

end_of_app:

	// De Initialize the Library
	DeInitLibrary();
			
	//removing lock file
	ret = remove("/var/lock/LOCK..E_CAM33");
	if(ret <0)
	{
		printf("No such file or directory");
	}
	
	// Exiting from the Application
	return 0;

}// end of main loop

int find_lock_file()
{
	int pid_number;
	int save_pid;
	FILE *proc_file;
	char filename[32];

	pid_number = (int) getpid();

	//open the lock file in read mode
	
	lockfile = fopen("/var/lock/LOCK..E_CAM33","r");
	if(lockfile == NULL)
	{
		//create the lock file in write mode
		lockfile = fopen("/var/lock/LOCK..E_CAM33", "w+");
		if(lockfile == NULL)
		{
			printf("unable to create the lock file \n");
			return 0;
		}
		fprintf(lockfile,"%d\n",pid_number);
	}
	else
	{
		fscanf(lockfile,"%d",&save_pid);
		sprintf(filename, "/proc/%d",save_pid);
		proc_file = fopen(filename,"r");
		if(proc_file == NULL)
		{
			//open the lock file in write mode
			lockfile = fopen("/var/lock/LOCK..E_CAM33", "w+");
			if(lockfile == NULL)
			{
				printf("unable to open the lock file \n");
				return 0;
			}
			fprintf(lockfile,"%d\n",pid_number);
		}
		else
		{	
			printf("** WARNING **: Another instance of e_cam33 UVC application(%d) is already running! \n",save_pid);
			return 0;
		}		
		
	}
	fclose(lockfile);	
	return 1;
}



