//e_cam33_lib.c ---- e_cam33 library source file

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <fcntl.h>      
#include <errno.h>
#include <malloc.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <linux/kernel.h>
//#include <linux/videodev.h>
#include <linux/videodev2.h>


#include "e_cam33_lib.h"

//Global Variable Declarations
//char *device_name = NULL;
static int fd = -1;
FILE *fstream = NULL;
char g_buff[256];
BOOL g_debugmode = FALSE;

#define V4L2_NAME_MAX				512			// Example: video0
#define DEVICE_NAME_MAX				(6+V4L2_NAME_MAX)	// Example: /dev/video0
#define PRODUCT_NAME_MAX			1024			// Example: e-con System's 3MP AF Camera
#define CAM_LIST_MAX				128
#define FILE_NAME_MAX				2048	

typedef struct __usb_webcam_info
{
	char	v4l2_name[V4L2_NAME_MAX];
	char	device_name[DEVICE_NAME_MAX];
	unsigned short	vendor_id;
	unsigned short	product_id;
	unsigned short	revision_num;
	char	product_name[PRODUCT_NAME_MAX];
} usb_webcam;

usb_webcam *pcurrent_webcam;

/* UVC XU controls */
struct uvc_xu_control_info xu_ctrls[] = {

	/* REGISTER CONTROL */
	{
    		.entity   = UVC_GUID_E_CAM33_WEBCAM,
    		.selector = XU_HW_CONTROL_REGISTER,
		.index    = 3,
		.size     = 4,
		.flags    = UVC_CONTROL_SET_CUR | UVC_CONTROL_GET_CUR | UVC_CONTROL_GET_MIN | UVC_CONTROL_GET_MAX | UVC_CONTROL_GET_RES | UVC_CONTROL_GET_DEF
  	},

	/* FOCUS CONTROL */
	{
    		.entity   = UVC_GUID_E_CAM33_WEBCAM,
    		.selector = XU_HW_CONTROL_SENSOR,
		.index    = 15,
		.size     = 9,
		.flags    = UVC_CONTROL_SET_CUR | UVC_CONTROL_GET_CUR | UVC_CONTROL_GET_MIN | UVC_CONTROL_GET_MAX | UVC_CONTROL_GET_RES | UVC_CONTROL_GET_DEF
  	}	
};

struct uvc_xu_control_mapping xu_mappings[] = {

	/* MAPPING FOR REGISTER CONTROL */
	{
    		.id        = V4L2_CID_REG_MODE_E_CAM33_WEBCAM,
    		.name      = "e-cam33 Register Control",
    		.entity    = UVC_GUID_E_CAM33_WEBCAM,
    		.selector  = XU_HW_CONTROL_REGISTER,
    		.size      = 8,
    		.offset    = 0,
    		.v4l2_type = V4L2_CTRL_TYPE_INTEGER,
    		.data_type = UVC_CTRL_DATA_TYPE_UNSIGNED
  	},
	/* MAPPING FOR FOCUS CONTROL */
	{
    		.id        = V4L2_CID_AUTO_FOCUS_MODE_E_CAM33_WEBCAM,
    		.name      = "e-cam33 FOCUS Control",
    		.entity    = UVC_GUID_E_CAM33_WEBCAM,
    		.selector  = XU_HW_CONTROL_SENSOR,
    		.size      = 8,
    		.offset    = 0,
    		.v4l2_type = V4L2_CTRL_TYPE_INTEGER,
    		.data_type = UVC_CTRL_DATA_TYPE_UNSIGNED
  	}
};

// UVC XU API Function Definitions
/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRARY API				
 *  Name	:	InitLibrary
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	: 	to initializes the e_cam33 library  	
 ************************************************************************************************************/
INT32 InitLibrary(VOID)
{
	int i = 0;
	int ret = 0;
	
	time_t ltime; /* calendar time */
	ltime = time(NULL); /* get current cal time */

	open_logfile();
	debug_message("*****************************************\n");
	debug_message("   ***UVC WEBCAM APPLICATION STARTS***   \n");
	sprintf(g_buff,"       AT %s",asctime(localtime(&ltime)));
	debug_message(g_buff);
	debug_message("*****************************************\n");

	// Get the details of the USB Web Cam Devices connected. Then use the first e-cam33 USB Web Cam for the UVC XU Control
	get_usb_webcam_details();
	pcurrent_webcam = (usb_webcam *)find_first_e_cam33_webcam();
	//If e_cam33 device is not match with given PID and VID
	if(pcurrent_webcam == NULL)
	{
		printf("No device is found with given VID(0x2560) and PID(0xc033)\n");
		ret=FAILURE;
		goto end_of_InitLibrary;
	}
	if(pcurrent_webcam->device_name==NULL)
	{
		debug_message("Failed to read usb webcam of E-con Systems \n");
		ret=FAILURE;
		goto end_of_InitLibrary;
	}
	printf("Selected Device for UVC XU Control is %s \n",pcurrent_webcam->device_name);

	//Open the video device
	ret = open_device(pcurrent_webcam->device_name);
	if(ret<0)
	{
		debug_message("Failed to open video device \n");
		goto end_of_InitLibrary;
	}

	// Add the UVC XU Control for Register
	for ( i=0; i<(LENGTH_OF(xu_ctrls)-1); i++ )
	{
  		sprintf(g_buff, "adding control for %d\n", i);
		debug_message(g_buff);
		errno=0;
	    	if ((ret=ioctl(fd, UVCIOC_CTRL_ADD, &xu_ctrls[i])) < 0 )
		{
      			if ( errno != EEXIST )
			{sprintf(g_buff, "FUNC %s(): LINE %d: Sensor Mode Selection for 9 Bytes Data Length in Sensor Selector Control \n",__FUNCTION__,__LINE__);
	debug_message(g_buff);
        			sprintf(g_buff,"uvcioc ctrl add error: errno=%d (retval=%d) %s \n", errno, ret,strerror(errno));
				debug_message(g_buff);
				goto end_of_InitLibrary;
      			}
			else
			{
				ret = 0;
	       			sprintf(g_buff,"control %d already exists\n", i);
				debug_message(g_buff);
      			}
    		}
		else
		{
			sprintf(g_buff,"FUNC %s(): LINE %d: Success in Adding the UVC Control for %d \n",__FUNCTION__,__LINE__,i);
			debug_message(g_buff);
		}
  	} 

	// Map the UVC XU Control for Register
  	for ( i=0; i<(LENGTH_OF(xu_mappings)-1); i++ ) 
	{
    		sprintf(g_buff, "mapping controls for %s\n", xu_mappings[i].name);
		debug_message(g_buff);
    		errno=0;
    		if ((ret=ioctl(fd, UVCIOC_CTRL_MAP, &xu_mappings[i])) < 0)
		{
      			if (errno != EEXIST)
			{
        			sprintf(g_buff,"uvcioc ctrl map error: errno=%d (retval=%d) %s \n", errno, ret,strerror(errno));
				debug_message(g_buff);
				goto end_of_InitLibrary;
      			}
			else
			{
				ret = 0;
			       	sprintf(g_buff, "mapping %d already exists\n", i);
				debug_message(g_buff);
      			}
    		}
		else
		{
			sprintf(g_buff, "FUNC %s(): LINE %d: Success in Mapping the UVC Control for %d \n",__FUNCTION__,__LINE__,i);
			debug_message(g_buff);
		}
  	}

	sprintf(g_buff, "FUNC %s(): LINE %d: Sensor Mode Selection for 9 Bytes Data Length in Sensor Selector Control \n",__FUNCTION__,__LINE__);
	debug_message(g_buff);

	//Select the Sensor Mode as 9 byte transfer
	ret = sensor_mode_selection();
	if(ret<0)
	{
		debug_message("Failed to do sensor mode selection \n");
		goto end_of_InitLibrary;
	}

	// Add the UVC XU control for CONTROL SENSOR
	i=(LENGTH_OF(xu_mappings)-1);
	if ((ret=ioctl(fd, UVCIOC_CTRL_ADD, &xu_ctrls[i])) < 0 )
	{
      		if ( errno != EEXIST )
		{
        		sprintf(g_buff, "uvcioc ctrl add error: errno=%d (retval=%d) %s \n", errno, ret,strerror(errno));
			debug_message(g_buff);
			goto end_of_InitLibrary;
      		}
		else
		{
			ret = 0;
        		sprintf(g_buff, "control %d already exists\n", i);
			debug_message(g_buff);
      		}
    	}
	else
	{
		sprintf(g_buff,"FUNC %s(): LINE %d: Success in Adding the UVC Control for %d \n",__FUNCTION__,__LINE__,i);
		debug_message(g_buff);
	}

	// Add the Map for the added UVC XU Control for Control Sensor
	i=(LENGTH_OF(xu_mappings)-1);
	if ((ret=ioctl(fd, UVCIOC_CTRL_MAP, &xu_mappings[i])) < 0)
	{
      		if (errno != EEXIST)
		{
        		sprintf(g_buff, "uvcioc ctrl map error: errno=%d (retval=%d) %s \n", errno, ret,strerror(errno));
			debug_message(g_buff);
			goto end_of_InitLibrary;
      		}
		else
		{
			ret = 0;
		        sprintf(g_buff,"mapping %d already exists\n", i);
			debug_message(g_buff);
      		}
    	}
	else
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: Success in Mapping the UVC Control for %d \n",__FUNCTION__,__LINE__,i);
		debug_message(g_buff);
	}
		
end_of_InitLibrary:
	if(ret<0)
	{
		close_device();
		close_logfile();
		return FAILURE;
	}
	return SUCCESS;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRARY/CONTROL API				
 *  Name	:	AFFreezeControl
 *  Parameter1	:	INT32
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	: 	to control the AUTOFOCUS Freeze functionality of the Avior
 ************************************************************************************************************/
INT32 AFFreezeControl(INT32 AFcontrol)
{
	INT32 AFStatus,ret;
	switch(AFcontrol)
	{
		case AF_CONT_FREEZE:
		{
			if(auto_focus_control(AUTO_FOCUS_CNT_AUTO_FOCUS_FREEZE) == SUCCESS)
			{
				usleep(DELAY);
				ret = GetAFStatus(&AFStatus);
				if(ret == FAILURE)
				{
					debug_message("Auto Focus Toggle Fails at Get AF status \n");
					return FAILURE;
				}
				if(AFStatus == AF_CONT_FROZEN)
				{
					return AF_CONT_FREEZE;
				}
				else
					return FAILURE;
			}
			else
			{
				return FAILURE;
			}
		}
		break;
		case AF_CONT_UNFREEZE:
		{
			if(auto_focus_control(AUTO_FOCUS_CNT_AUTO_FOCUS_UNFREEZE_AND_CONTINUE) == SUCCESS)
			{
				usleep(DELAY);
				ret = GetAFStatus(&AFStatus);
				if(ret == FAILURE)
				{
					debug_message("Auto Focus Toggle Fails at Get AF status \n");
					return FAILURE;
				}
				if(AFStatus == AF_CONT_FOCUSED)
				{
					return AF_CONT_UNFREEZE;
				}
				else
					return FAILURE;
			}
			else
			{
				return FAILURE;
			}
		}
		break;
		case MANUAL_FOCUS_MODE:
		{
			if(auto_focus_control(AUTO_FOCUS_CNT_MANUAL_FOCUS_SELECT) == SUCCESS)
			{
				usleep(DELAY);
				ret = GetAFStatus(&AFStatus);
				if(ret == FAILURE)
				{
					debug_message("Auto Focus Toggle Fails at Get AF status \n");
					return FAILURE;
				}
				if(AFStatus == MANUAL_FOCUS)
				{
					return MANUAL_FOCUS_MODE;
				}
				else
					return FAILURE;
			}
			else
			{
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS:
		{
			if(auto_focus_control(AUTO_FOCUS_CNT_AUTO_FOCUS_SELECT) == SUCCESS)
			{
				usleep(DELAY);
				ret = GetAFStatus(&AFStatus);
				if(ret == FAILURE)
				{
					debug_message("Auto Focus Toggle Fails at Get AF status \n");
					return FAILURE;
				}
				if(AFStatus == AF_CONT_FOCUSING)
				{
					return AUTO_FOCUS;
				}
				else
					return FAILURE;
			}
			else
			{
				return FAILURE;
			}
		}
		break;
		case SINGLE_TRIG_AF:
		{
			if(auto_focus_control(AUTO_FOCUS_SINGLE_TRIGGER_FOCUS_SELECT) == SUCCESS)
			{
				usleep(DELAY);
				ret = GetAFStatus(&AFStatus);
				if(ret == FAILURE)
				{
					debug_message("Auto Focus Toggle Fails at Get AF status \n");
					return FAILURE;
				}
				if((AFStatus == AF_SINGLE_TRIGGER_FOCUSED) || (AFStatus == AF_SINGLE_TRIGGER_FOCUSING))
				{
					return SINGLE_TRIG_AF;
				}
				else
					return FAILURE;
			}
			else
			{
				return FAILURE;
			}
		}
		break;
		default:
		{
			return FAILURE;
		}
		break;
	}
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRARY/CONTROL API 					
 *  Name	:	GetAFStatus
 *  Parameter1	:	INT32 *
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	:       to read the AutoFocus Freeze status	
 ************************************************************************************************************/
INT32 GetAFStatus(INT32 *AFStatus)
{
	int status;

	status = auto_focus_control(AUTO_FOCUS_CNT_GET_FOCUS_STATUS);
	//printf("status is :%x\n",status);
	//sprintf(g_buff,"status:%x\n",status);
	//debug_message(g_buff);
	if((status & 0x80) == 0x80)
	{
		//Step State Failed
		*AFStatus = FAILURE;
		return FAILURE;
	}
	if((status & 0x0C) == 0x08)
	{
		//AUTO FOCUS MODE
		if((status & 0x03) == 0x01)
		{
			debug_message("AFStatus:In Auto focus mode, Focusing  \n");
			*AFStatus = AF_CONT_FOCUSING;
			return SUCCESS;
		}
		if((status & 0x03) == 0x02)
		{
			debug_message("AFStatus:In Auto focus mode, Focused \n");
			*AFStatus = AF_CONT_FOCUSED;
			return SUCCESS;
		}
		if((status & 0x03) == 0x03)
		{
			debug_message("AFStatus:In Auto focus mode, Freezed \n");
			*AFStatus = AF_CONT_FROZEN;
			return SUCCESS;
		}
	}	
	else if((status & 0x0C) == 0x04)
	{
		//SINGLE TRIGGER AUTO FOCUS
		if((status & 0x03) == 0x01)
		{
			debug_message("AFStatus:In Single Trigger Auto focus mode, Focusing  \n");
			*AFStatus = AF_SINGLE_TRIGGER_FOCUSING;
			return SUCCESS;
		}
		if((status & 0x03) == 0x02)
		{
			debug_message("AFStatus:In Single Trigger Auto focus mode, Focused \n");
			*AFStatus = AF_SINGLE_TRIGGER_FOCUSED;
			return SUCCESS;
		}
		if((status & 0x03) == 0x03)
		{
			debug_message("AFStatus:In Single Trigger Auto focus mode, Freezed \n");
			*AFStatus = AF_SINGLE_TRIGGER_FROZEN;
			return SUCCESS;
		}
		return SUCCESS;
	}	
	else if((status & 0x0C) == 0x0C)
	{
		//MANUAL FOCUS MODE
		debug_message("AFStatus:In Manual Mode \n");
		*AFStatus = MANUAL_FOCUS;
		return SUCCESS;
	}
	else
	{
		*AFStatus = FAILURE;
		return FAILURE;
	}
	
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRAY/CONTROL API 					
 *  Name	:	SetManualFocusPosition
 *  Parameter1	:	INT16
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	:       to set the Manual Focus Position	
 ************************************************************************************************************/
INT32 SetManualFocusPosition(INT16 position)
{
	unsigned char data[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static struct  uvc_xu_control xu_ctrl_data ={
		/* DATA FOR SENSOR CONTROL */
		.unit		= 4,
		.selector	= XU_HW_CONTROL_SENSOR,
		.size 		= 9,
		.data		= NULL
	};
	int ret=0;
	xu_ctrl_data.data = data;
	xu_ctrl_data.size = (sizeof(data));
	sprintf(g_buff,"FUNC %s(): LINE %d: xu_ctrl_data.size is %d \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.size);
	debug_message(g_buff);
	sprintf(g_buff,"FUNC %s(): LINE %d: position is %d \n",__FUNCTION__,__LINE__,position);
	debug_message(g_buff);

	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_WRITE_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (SENSOR_MANUAL_FOCUS_REG_ADDR & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (SENSOR_MANUAL_FOCUS_REG_ADDR >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = position; // Data for manual focus position
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_SET,&xu_ctrl_data);
	if(ret<0)
	{
   		sprintf(g_buff,"FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		sprintf(g_buff,"FUNC %s(): LINE %d: current control value is %d (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[3],ret);
		debug_message(g_buff);
	}
	return SUCCESS;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRAY/CONTROL API 				
 *  Name	:	GetManualFocusPosition
 *  Parameter1	:	INT16 *	
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	:       to get the Manual Focus Position
 ************************************************************************************************************/
INT32 GetManualFocusPosition(INT16 *position)
{
	unsigned char data[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static struct  uvc_xu_control xu_ctrl_data ={
		/* DATA FOR SENSOR CONTROL */
		.unit		= 4,
		.selector	= XU_HW_CONTROL_SENSOR,
		.size 		= 9,
		.data		= NULL
	};
	int ret=0;
	xu_ctrl_data.data = data;
	xu_ctrl_data.size = (sizeof(data));
	sprintf(g_buff,"FUNC %s(): LINE %d: xu_ctrl_data.size is %d \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.size);
	debug_message(g_buff);

	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_READ_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (SENSOR_MANUAL_FOCUS_REG_ADDR & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (SENSOR_MANUAL_FOCUS_REG_ADDR >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = 0x00; 
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_SET,&xu_ctrl_data);
	if(ret<0)
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		//printf("FUNC %s(): LINE %d: current control value is %d (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[3],ret);
	}
	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_READ_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (SENSOR_MANUAL_FOCUS_REG_ADDR & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (SENSOR_MANUAL_FOCUS_REG_ADDR >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = 0x00; 
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_GET,&xu_ctrl_data);
	if(ret<0)
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: Manual Focus Position is 0x%02X \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[6]);
		debug_message(g_buff);
		*position = (unsigned short int)xu_ctrl_data.data[6];
		return SUCCESS;
	}
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRAY/CONTROL API 					
 *  Name	:	GetSensorId
 *  Parameter1	:	INT32
 *  Parameter2	:	
 *  Returns	:	INT32
 *  Description	:       to read the sensor id	 	
 ************************************************************************************************************/

INT32 GetSensorId(INT32 *SensorID)
{
	int sensorid1,sensorid2;

	sensorid1 = diagnostic_control(DIAGNOSTIC_CNT_READ_SENSOR_ID1);
	sensorid2 = diagnostic_control(DIAGNOSTIC_CNT_READ_SENSOR_ID2);

	if((sensorid1 != FAILURE) && (sensorid2 != FAILURE))
	{
		*SensorID = (sensorid1 << 8) | sensorid2;
		return SUCCESS;
	}
	else
	{
		return FAILURE;
	}
}


/************************************************************************************************************
 *  
 *  MODULE TYPE	:	LIBRAY API 				
 *  Name	:	DeInitLibrary
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	void
 *  Description	:       to release all the extension unit objects and other internal library objects		
 ************************************************************************************************************/
void DeInitLibrary(VOID)
{
	close_device();
	close_logfile();
	return;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUCNTION 				
 *  Name	:	auto_focus_control
 *  Parameter1	:	int
 *  Parameter2	:	
 *  Returns	:	int
 *  Description	:       Control function for autofocus mode control
	
 ************************************************************************************************************/
int auto_focus_control(int focus_option)
{
	unsigned char focus_status;
	switch(focus_option)
	{
		case AUTO_FOCUS_CNT_AUTO_FOCUS_FREEZE:
		{
			if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR,DATA_FOR_FREEZE_IN_AUTO_FOCUS_MODE)) != FAILURE)
			{
				return SUCCESS;
			}
			else
			{
				sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS_CNT_AUTO_FOCUS_UNFREEZE_AND_CONTINUE:
		{
			if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA_FOR_UNFREEZE_AND_CONTINUE__AUTO_FOCUS_MODE)) != FAILURE)
			{
				return SUCCESS;
			}
			else
			{
				sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS_CNT_MANUAL_FOCUS_SELECT:
		{
			if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA1_FOR_SETTING_MANUAL_FOCUS_MODE)) != FAILURE)
			{
				if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA2_FOR_SETTING_MANUAL_FOCUS_MODE)) != FAILURE)
				{
					return SUCCESS;
				}
				else
				{
					sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
					debug_message(g_buff);
					return FAILURE;
				}
			}
			else
			{
				sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS_CNT_AUTO_FOCUS_SELECT:
		{
			if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA1_FOR_SETTING_AUTO_FOCUS_MODE)) != FAILURE)
			{
				if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA2_FOR_SETTING_AUTO_FOCUS_MODE)) != FAILURE)
				{
					return SUCCESS;
				}
				else
				{
					sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
					debug_message(g_buff);
					return FAILURE;
				}
			}
			else
			{
				sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS_SINGLE_TRIGGER_FOCUS_SELECT:
		{
			if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA1_FOR_SETTING_AUTO_FOCUS_MODE)) != FAILURE)
			{
				//sleep(2);
				if((write_sensor_reg(SENSOR_FOCUS_SELECTION_REG_ADDR, DATA_FOR_SETTING_SINGLE_TRIGGER_MODE)) != FAILURE)
				{
					return SUCCESS;
				}
				else
				{
					sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
					debug_message(g_buff);
					return FAILURE;
				}
			}
			else
			{
				sprintf(g_buff,"Error in writing the sensor register:%x\n",SENSOR_FOCUS_SELECTION_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		case AUTO_FOCUS_CNT_GET_FOCUS_STATUS:
		{
			if((read_sensor_reg(SENSOR_FOCUS_STATUS_REG_ADDR, &focus_status)) != FAILURE)
			{
				return (int)focus_status;
			}
			else
			{
				sprintf(g_buff,"Error in reading the sensor register:%x\n",SENSOR_FOCUS_STATUS_REG_ADDR);
				debug_message(g_buff);
				return FAILURE;
			}
		}
		break;
		default:
		{
			sprintf(g_buff,"FUNC %s(): LINE %d: %d is invalid option \n",__FUNCTION__,__LINE__,focus_option);
			debug_message(g_buff);
			return FAILURE;
		}
		break;		
	}// End of switch case
	return SUCCESS;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUCNTION 				
 *  Name	:	diagnostic_control
 *  Parameter1	:	int
 *  Parameter2	:	
 *  Returns	:	int
 *  Description	:       Control function for reading the Sensor ID
 ************************************************************************************************************/
int diagnostic_control(int diagnostic_option)
{
	unsigned char sensor_id1_value, sensor_id2_value;
	switch(diagnostic_option)
	{
		case DIAGNOSTIC_CNT_READ_SENSOR_ID1:
		{

			if((read_sensor_reg(SENSOR_ID1_REG_ADDR, &sensor_id1_value)) != FAILURE) 
			{
				return (int)sensor_id1_value;
			}
		}
		break;
		case DIAGNOSTIC_CNT_READ_SENSOR_ID2:
		{
			if((read_sensor_reg(SENSOR_ID2_REG_ADDR, &sensor_id2_value)) != FAILURE)
			{
				return (int)sensor_id2_value;
			}
		}
		break;
		default:
		{
			sprintf(g_buff, "FUNC %s(): LINE %d: %d is invalid option \n",__FUNCTION__,__LINE__,diagnostic_option);
			debug_message(g_buff);						
		}
		break;	
	} // end of switch 
	return FAILURE;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	read_sensor_reg
 *  Parameter1	:	unsigned int
 *  Parameter2	:	unsigned char *
 *  Returns	:	int
 *  Description	: 	to read the sensor register address(argument 1)
 ************************************************************************************************************/

int read_sensor_reg(unsigned int reg_address, unsigned char *reg_data)
{
	unsigned char data[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static struct  uvc_xu_control xu_ctrl_data ={
		/* DATA FOR SENSOR CONTROL */
		.unit		= 4,
		.selector	= XU_HW_CONTROL_SENSOR,
		.size 		= 9,
		.data		= NULL
	};
	int ret=0;
	xu_ctrl_data.data = data;
	xu_ctrl_data.size = (sizeof(data));
	//sprintf(g_buff,"FUNC %s(): LINE %d: xu_ctrl_data.size is %d \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.size);
	//debug_message(g_buff);
	
	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_READ_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (reg_address & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (reg_address >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = 0x00; 
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_SET,&xu_ctrl_data);
	if(ret<0)
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		//printf("FUNC %s(): LINE %d: current control value is %d (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[3],ret);
	}

	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_READ_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (reg_address & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (reg_address >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = 0x00; 
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_GET,&xu_ctrl_data);
	if(ret<0)
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: 0x%X Reg value is 0x%02X \n",__FUNCTION__,__LINE__,reg_address,(unsigned int)xu_ctrl_data.data[6]);
		debug_message(g_buff);
		*reg_data = xu_ctrl_data.data[6];
	}
	return SUCCESS;
}


/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	write_sensor_reg
 *  Parameter1	:	unsigned int
 *  Parameter2	:	unsigned char
 *  Returns	:	int
 *  Description	: 	to write the sensor register address(argument 1)
 ************************************************************************************************************/

int write_sensor_reg(unsigned int reg_address, unsigned char reg_data)
{
	unsigned char data[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
	static struct  uvc_xu_control xu_ctrl_data ={
		/* DATA FOR SENSOR CONTROL */
		.unit		= 4,
		.selector	= XU_HW_CONTROL_SENSOR,
		.size 		= 9,
		.data		= NULL
	};
	int ret=0;
	xu_ctrl_data.data = data;
	xu_ctrl_data.size = (sizeof(data));
	//sprintf(g_buff,"FUNC %s(): LINE %d: xu_ctrl_data.size is %d \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.size);
	//debug_message(g_buff);

	// Control Type
	xu_ctrl_data.data[0] = SENSOR_DEVICE_ADDRESS_FOR_WRITE_OPERATION;
	// Sensor Sub Address Byte Number
	xu_ctrl_data.data[1] = 0x02;	// Sensor Sub Address (or Register Address within the Sensor) is 16bit
	// Sensor value by number
	xu_ctrl_data.data[2] = 0x01;	// Sensor Data is 8bit
	// Sensor Address
	xu_ctrl_data.data[3] = (reg_address & 0xFF); // LSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[4] = (reg_address >> 8); // MSB of 16 bit Sensor Sub Address
	xu_ctrl_data.data[5] = 0x00; // Always 0x00 for 16bit Sensor Sub Address
	// Sensor Data
	xu_ctrl_data.data[6] = reg_data; // Data for flip bit
	xu_ctrl_data.data[7] = 0x0; // Always 0x00 for 8bit Sensor Data
	xu_ctrl_data.data[8] = 0x0; // Always 0x00 for 8bit Sensor Data
	ret = ioctl(fd,UVCIOC_CTRL_SET,&xu_ctrl_data);
	if(ret<0)
	{
   		sprintf(g_buff,"FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		sprintf(g_buff,"FUNC %s(): LINE %d: current control value is %x (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[6],ret);
		debug_message(g_buff);
	}
	return SUCCESS;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION			
 *  Name	:	sensor_mode_selection
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	int
 *  Description	: 	to select the sensor control as 9 byte mode	
 ************************************************************************************************************/
int sensor_mode_selection(void)
{
	unsigned char data[]={0x00,(SENSOR_MODE_SELECT_REG_ADDR & 0xFF), (SENSOR_MODE_SELECT_REG_ADDR>>8), 0x09};
	static struct  uvc_xu_control xu_ctrl_data ={
		/* DATA FOR SENSOR MODE SEELCT ( 9 Bytes Data for SENSOR selector control) */
		.unit		= 4,
		.selector	= XU_HW_CONTROL_REGISTER,
		.size 		= 4,
		.data		= NULL
	};
	int ret=0;
	xu_ctrl_data.data = data;
	xu_ctrl_data.size = (sizeof(data));
	xu_ctrl_data.data[0] |= REG_CONTROL_WRITE_REGISTER;

	ret = ioctl(fd,UVCIOC_CTRL_SET,&xu_ctrl_data);
	if(ret<0)
	{
        	sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		//printf("FUNC %s(): LINE %d: current control value is %d (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[3],ret);
	}
	xu_ctrl_data.data[0] = 0x00;
	xu_ctrl_data.data[3] = 0x00;
	ret = ioctl(fd,UVCIOC_CTRL_GET,&xu_ctrl_data);
	if(ret<0)
	{
        	sprintf(g_buff, "FUNC %s(): LINE %d: failed to get control value.errno=%d (retval=%d) %s \n",__FUNCTION__,__LINE__,errno, ret,strerror(errno));
		debug_message(g_buff);
		return FAILURE;
	}
	else
	{
		sprintf(g_buff, "FUNC %s(): LINE %d: current control value is %d (return value %d) \n",__FUNCTION__,__LINE__,(unsigned int)xu_ctrl_data.data[3],ret);
		debug_message(g_buff);
	}
	return SUCCESS;
	
}


/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	open_device
 *  Parameter1	:	char *
 *  Parameter2	:	
 *  Returns	:	int
 *  Description	: 	to open the v4l2 camera driver
 ************************************************************************************************************/
int open_device(char *dev_name)
{
	struct stat st;							//to get the file system and file information
	if (-1 == stat (dev_name, &st)) 	
	{					                
		sprintf(g_buff, "%s Cannot identify %d, %s\n", dev_name, errno, strerror (errno));
		debug_message(g_buff);
		return -1;
	}	 
	if (!S_ISCHR (st.st_mode)) 
	 {			
		sprintf(g_buff, "%s is no device\n", dev_name);			
		debug_message(g_buff);
		return -1;
	 }
	 fd = open(dev_name,O_RDWR,0);			 //open the device as file in read only mode
	 if (-1 == fd) 
	 {
		debug_message("cann't open the driver\n");
		sprintf (g_buff, "Cannot open %s%d%s\n", dev_name, errno, strerror (errno));
		debug_message(g_buff);
		return -1;
	 }
	 return 0;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION			
 *  Name	:	close_device
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	void
 *  Description	: 	to perform the close operation of v4l2 driver
 ************************************************************************************************************/
void close_device(void)
{
	if(fd>0)
	{
		close(fd);
		fd=-1;
	}
	return ;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	open_logfile
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	void
 *  Description	: 	to open the log file for saving the debug message	
 ************************************************************************************************************/
void open_logfile()
{
	fstream = fopen("./e_cam33_log.txt", "a+");
	if(fstream == NULL)
	{
		printf("Unable to create the log file\n");
		g_debugmode = FALSE;
	}
	else
	{
		g_debugmode = TRUE;
	}
	return;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	close_logfile
 *  Parameter1	:	void
 *  Parameter2	:	
 *  Returns	:	void
 *  Description	: 	to close the log file
 ************************************************************************************************************/
void close_logfile()
{
	if(g_debugmode)
	{
		if(fstream != NULL)
		{
			if(fclose(fstream))
			{
				debug_message("The file e_cam33_log.txt was not closed\n");
			}
			g_debugmode = FALSE;
			fstream = NULL;
		}
	}
	return;
}

/************************************************************************************************************
 *  
 *  MODULE TYPE	:	FUNCTION					
 *  Name	:	debug_message
 *  Parameter1	:	char*
 *  Parameter2	:	
 *  Returns	:	void
 *  Description	: 	to write the log message in to the log file
 ************************************************************************************************************/
void debug_message(char *msg)
{
	if(g_debugmode)
	{
		if(fstream != NULL)
		{
			fprintf(fstream, "%s", msg);
		}
	}
	else
	{
		fprintf(stdout, "%s", msg);	
	}	
	return;
}

