// e_cam33_lib.h ----  e-cam33 library header file

#ifndef __E_CAM33_LIB_H__
#define __E_CAM33_LIB_H__

#include <stdbool.h>
#include "uvcvideo.h"
#include "e_cam33_api.h"

#define ABS(a) (((a) < 0) ? -(a) : (a))
#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#define MAX(a, b) (((a) > (b)) ? (a) : (b))
#define LENGTH_OF(x) (sizeof(x)/sizeof(x[0]))


//Guid of E-CAM33 extension unit : {A917C75D-4119-11DA-AE0E-000D56AC7B4C} 
#define UVC_GUID_E_CAM33_WEBCAM				{0x5D, 0xC7, 0x17, 0xA9, 0x19, 0x41, 0xDA, 0x11, 0xAE, 0x0E, 0x00, 0x0D, 0x56, 0xAC, 0x7B, 0x4C}

// V4L2 ID FOR UVC XU CONTROLS
#define V4L2_CID_AUTO_FOCUS_MODE_E_CAM33_WEBCAM 	0x0A046D17
#define V4L2_CID_REG_MODE_E_CAM33_WEBCAM      		0x0A046D07

#define REG_CONTROL_REG_TYPE_CHIP_REGISTER		(0x0 << 0) 
#define REG_CONTROL_REG_TYPE_SENSOR_REGISTER		(0x1 << 0)
#define REG_CONTROL_REG_TYPE_MASK			(0xF << 0)
#define REG_CONTROL_REG_VALUE_TYPE_BYTE			(0x0 << 4)
#define REG_CONTROL_REG_VALUE_TYPE_WORD			(0x1 << 4)
#define REG_CONTROL_REG_VALUE_MASK			(0x1 << 4)
#define REG_CONTROL_READ_REGISTER			(0x0 << 5)
#define REG_CONTROL_WRITE_REGISTER			(0x1 << 5)
#define REG_CONTROL_READ_WRITE_MASK			(0x1 << 5)

#define SENSOR_DEVICE_ADDRESS				0x3C  // 7bit address
#define SENSOR_DEVICE_ADDRESS_FOR_READ_OPERATION	(SENSOR_DEVICE_ADDRESS <<1)
#define SENSOR_DEVICE_ADDRESS_FOR_WRITE_OPERATION	((SENSOR_DEVICE_ADDRESS <<1) | 0x01)
#define SENSOR_MODE_SELECT_REG_ADDR			0x8102
#define SENSOR_FOCUS_SELECTION_REG_ADDR			0x3F00
#define SENSOR_FOCUS_STATUS_REG_ADDR			0x3F01
#define SENSOR_MANUAL_FOCUS_REG_ADDR			0x3F06
#define SENSOR_ID1_REG_ADDR				0x300A
#define SENSOR_ID2_REG_ADDR				0x300B

#define DATA_FOR_FREEZE_IN_AUTO_FOCUS_MODE		0x06
#define DATA_FOR_UNFREEZE_AND_CONTINUE__AUTO_FOCUS_MODE	0x07
#define DATA1_FOR_SETTING_MANUAL_FOCUS_MODE		0x08
#define DATA2_FOR_SETTING_MANUAL_FOCUS_MODE		0x05
#define DATA1_FOR_SETTING_AUTO_FOCUS_MODE		0x08
#define DATA2_FOR_SETTING_AUTO_FOCUS_MODE		0x04
#define DATA_FOR_SETTING_SINGLE_TRIGGER_MODE		0x03

#define MIN_MANUAL_FOCUS_POSITON			0
#define MAX_MANUAL_FOCUS_POSITION			255
#define DELAY						500000


enum auto_focus_control_options
{
	AUTO_FOCUS_CNT_AUTO_FOCUS_FREEZE=1,
	AUTO_FOCUS_CNT_AUTO_FOCUS_UNFREEZE_AND_CONTINUE,
	AUTO_FOCUS_CNT_MANUAL_FOCUS_SELECT,
	AUTO_FOCUS_CNT_AUTO_FOCUS_SELECT,
	AUTO_FOCUS_SINGLE_TRIGGER_FOCUS_SELECT,
	AUTO_FOCUS_CNT_GET_FOCUS_STATUS
};

enum diagnostic_control_options
{
	DIAGNOSTIC_CNT_READ_SENSOR_ID1=1,
	DIAGNOSTIC_CNT_READ_SENSOR_ID2
};

enum e_cam33_selector
{
	XU_HW_CONTROL_FLIP=1,
	XU_HW_CONTROL_AUTO_FOCUS,
	XU_HW_CONTROL_AUDIO_STREAMING_DETECTION,
	XU_HW_CONTROL_REGISTER,
	XU_HW_CONTROL_I2C_DEVICE,
	XU_HW_CONTROL_SENSOR_IDENTIFICATION,
	XU_HW_CONTROL_GET_FIRMWARE,
	XU_HW_CONTROL_GPIO,
	XU_HW_CONTROL_POWER_LINE_FREQUENCY,
	XU_HW_CONTROL_READ_SENSOR_2_2,
	XU_HW_CONTROL_WRITE_SENSOR_2_2,
	XU_HW_CONTROL_READ_SENSOR_3_1,
	XU_HW_CONTROL_WRITE_SENSOR_3_1,
	XU_HW_CONTROL_EXTERNAL_FLASH,
	XU_HW_CONTROL_READ_AND_WRITE_EXTERNAL_FLASH,
	XU_HW_CONTROL_SENSOR = 0x10,
	XU_HW_CONTROL_DMA_IIC = 0x18
};


// Function Prototype Declarations

char *find_first_e_cam33_webcam();
int auto_focus_control(int);
int diagnostic_control(int);
int read_sensor_reg(unsigned int, unsigned char *);
int write_sensor_reg(unsigned int, unsigned char);
int sensor_mode_selection(void);
int open_device(char *);
void close_device(void);
void open_logfile();
void close_logfile();
void debug_message(char *);


#endif





