#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <errno.h>


#define V4L2_NAME_MAX				512			// Example: video0
#define DEVICE_NAME_MAX				(6+V4L2_NAME_MAX)	// Example: /dev/video0
#define PRODUCT_NAME_MAX			1024			// Example: e-con System's 3MP AF Camera
#define CAM_LIST_MAX				128
#define FILE_NAME_MAX				2048	


#define SYSFS_VIDEO4LINUX			"/sys/class/video4linux/"
#define SYSFS_ENTRY_FOR_VIDEO_DEVICE_INFO	"/sys/class/video4linux/%s/device/uevent"	//For example, Here the %s is replaced by video0
#define SYSFS_ENTRY_FOR_VIDEO_DEVICE_NAME	"/sys/class/video4linux/%s/name"		//For example, Here the %s is replaced by video0

//USB WEBCAM
#define E_CON_SYSTEMS_3MP_AF_CAMERA_VENDOR_ID		0x2560
#define E_CON_SYSTEMS_3MP_AF_CAMERA_PRODUCT_ID		0xC033

typedef struct __usb_webcam_info
{
	char	v4l2_name[V4L2_NAME_MAX];
	char	device_name[DEVICE_NAME_MAX];
	unsigned short	vendor_id;
	unsigned short	product_id;
	unsigned short	revision_num;
	char	product_name[PRODUCT_NAME_MAX];
} usb_webcam;

usb_webcam	webcam_list[CAM_LIST_MAX];

usb_webcam *find_first_e_cam33_webcam(void);
int get_usb_webcam_details(void);


usb_webcam *find_first_e_cam33_webcam(void)
{
	int i=0;
	for(i=0;i<CAM_LIST_MAX;i++)
	{
		if((webcam_list[i].vendor_id == E_CON_SYSTEMS_3MP_AF_CAMERA_VENDOR_ID))
		{
			if((webcam_list[i].product_id == E_CON_SYSTEMS_3MP_AF_CAMERA_PRODUCT_ID))
			{
				return &webcam_list[i];
			}
		}	
	}
	return NULL;	
}

int get_usb_webcam_details(void)
{

	DIR *v4l_dir=NULL;
	struct dirent *dir_entry=NULL;
	int i=0;
	int device_count=0;
	char *filename=NULL;
	FILE *input=NULL;
	unsigned int file_size=0;
	char *data_buf=NULL;
	char *vendor_id_start=NULL,*vendor_id_end=NULL,*product_id_start=NULL,*product_id_end=NULL,*revision_num_start=NULL,*revision_num_end=NULL;
	char *vendor_id_string=NULL,*product_id_string=NULL,*revision_num_string=NULL;
	unsigned int vendor_id_length=0,product_id_length=0,revision_num_length=0;
	int ret=0;

	for(i=0;i<CAM_LIST_MAX;i++)
	{
		memset(&webcam_list[i],0,sizeof(usb_webcam));
	}

	// Go through all the v4l devices in sysfs
	v4l_dir = opendir(SYSFS_VIDEO4LINUX);
	if(v4l_dir)
	{
		while((dir_entry = readdir(v4l_dir)))
		{
			// Ignore the non-video devices
			if(strstr(dir_entry->d_name,"video") != dir_entry->d_name)
			{
				continue;
			}
			// Get the V4L device name
			memset(webcam_list[device_count].v4l2_name,0,V4L2_NAME_MAX);
			strcpy(webcam_list[device_count].v4l2_name,dir_entry->d_name);

			// Get the Device Node Name
			memset(webcam_list[device_count].device_name,0,DEVICE_NAME_MAX);
			sprintf(webcam_list[device_count].device_name,"/dev/%s",dir_entry->d_name);

			// Get the Product Name
			filename=malloc(FILE_NAME_MAX);
			if(filename==NULL)
			{
				printf("Failed to allocate memory for filename \n");
				goto end_of_app;
			}
			memset(filename,0,FILE_NAME_MAX);
			sprintf(filename,SYSFS_ENTRY_FOR_VIDEO_DEVICE_NAME,webcam_list[device_count].v4l2_name);
			input = fopen(filename,"r");
			if(input==NULL)
			{
				printf("Failed to open the file %s \n",filename);
				goto end_of_app;
			}
			memset(webcam_list[device_count].product_name,0,PRODUCT_NAME_MAX);
			fseek(input,0,SEEK_END);
			file_size = ftell(input);
			rewind(input);
			data_buf=malloc(file_size+1);
			if(data_buf==NULL)
			{
				printf("Failed to allocate memory for data buffer \n");
				goto end_of_app;
			}
			ret = fread(data_buf,1,file_size,input);
			if(ret != file_size)
			{
				// FIXME : The filesize is not matched with the actual file contents in the sysfs entry.
				//printf("Failed to read %d bytes from the file %s . ret is %d \n",file_size,filename,ret);
				//goto end_of_app;
			}
			
			// FIXME: The last character in the file for product name is always newline character. So skip this character.
			memcpy(webcam_list[device_count].product_name,data_buf,(ret-1));	
			if(input)
			{
				fclose(input);
				input=NULL;
			}	
			
			// Get the Vendor Id
			memset(filename,0,FILE_NAME_MAX);
			sprintf(filename,SYSFS_ENTRY_FOR_VIDEO_DEVICE_INFO,webcam_list[device_count].v4l2_name);
			input = fopen(filename,"r");
			if(input==NULL)
			{
				printf("Failed to open the file %s \n",filename);
				goto end_of_app;
			}
			fseek(input,0,SEEK_END);
			file_size = ftell(input);
			rewind(input);
			data_buf=malloc(file_size+1);
			if(data_buf==NULL)
			{
				printf("Failed to allocate memory for data buffer \n");
				goto end_of_app;
			}
			ret = fread(data_buf,1,file_size,input);
			
			if(ret != file_size)
			{
				// FIXME : The filesize is not matched with the actual file contents in the sysfs entry.
				//printf("Failed to read %d bytes from the file %s . ret is %d \n",file_size,filename,ret);
				//goto end_of_app;
			}
			vendor_id_start=strstr(data_buf,"PRODUCT=");
			if(vendor_id_start)
			{
				vendor_id_start=vendor_id_start+strlen("PRODUCT=");
				vendor_id_end = strchr(vendor_id_start,'/');
				if(vendor_id_end)
				{
					vendor_id_length=(unsigned int)(vendor_id_end)-(unsigned int)(vendor_id_start);
					vendor_id_string=malloc(vendor_id_length+1);
					if(vendor_id_string==NULL)
					{
						printf("Failed to allocate memory for vendor id string");
						goto end_of_app;
					}
					memset(vendor_id_string,0,vendor_id_length+1);
					strncpy(vendor_id_string,vendor_id_start,vendor_id_length);
					webcam_list[device_count].vendor_id = (unsigned short)strtoul(vendor_id_string,NULL,16);
					// Free the allocated memory
					if(vendor_id_string)
					{
						free(vendor_id_string);
						vendor_id_string=NULL;
					}
				}
				else
				{
					printf("Failed to Find the end of the vendor id string location \n");
					goto end_of_app;
				}
				// Get the Product Id
				product_id_start=vendor_id_end+1;
				if(product_id_start)
				{
					product_id_end = strchr(product_id_start,'/');
					if(product_id_end)
					{
						product_id_length = (unsigned int)(product_id_end)-(unsigned int)(product_id_start);
						product_id_string = malloc(product_id_length+1);
						if(product_id_string==NULL)
						{
							printf("Failed to allocate memory for product id string \n");
							goto end_of_app;
						}
						memset(product_id_string,0,product_id_length+1);
						strncpy(product_id_string,product_id_start,product_id_length);
						webcam_list[device_count].product_id = (unsigned short)strtoul(product_id_string,NULL,16);
						// Free the allocated memory
						if(product_id_string)
						{
							free(product_id_string);
							product_id_string=NULL;
						}
					}
					else
					{
						printf("Failed to Find the end of the product id string location \n");
						goto end_of_app;
					}
					
					// Get the Revision Number
					revision_num_start = product_id_end+1;
					if(revision_num_start)
					{
						revision_num_end = strchr(revision_num_start,'\n');
						if(revision_num_end)
						{
							revision_num_length = (unsigned int)(revision_num_end)-(unsigned int)(revision_num_start);
							revision_num_string = malloc(revision_num_length+1);
							if(revision_num_string==NULL)
							{
								printf("Failed to allocate memory for revision number string \n");
								goto end_of_app;
							}
							memset(revision_num_string,0,revision_num_length+1);
							strncpy(revision_num_string,revision_num_start,revision_num_length);
							webcam_list[device_count].revision_num = (unsigned short) strtoul(revision_num_string,NULL,16);
							// Free the allocated memory
							if(revision_num_string)
							{
								free(revision_num_string);
								revision_num_string=NULL;
							}
						}
						else
						{
							printf("Failed to Find the end of the revision number string location \n");
							goto end_of_app;
						}
						
					}
					else
					{
						printf("Failed to Find the start of the revision number string location \n");
						goto end_of_app;
					}
				}
				else
				{
					printf("Failed to Find the start of the product id string location \n");
					goto end_of_app;
				}
			}
			else
			{
				printf("Failed to Find the start of the vendor id string location \n");
				goto end_of_app;
			}
			if(data_buf)
			{
				free(data_buf);
				data_buf=NULL;
			}
			device_count++;
		}
	}
	else
	{
		printf("Failed to open the file %s \n",SYSFS_VIDEO4LINUX);
		goto end_of_app;
	}	

	for(i=0;i<device_count;i++)
	{
		printf(" Video Device %d details ==> \n",i);
		printf(" V4L2 device name 	: %s \n",webcam_list[i].v4l2_name);
		printf(" Device name		: %s \n",webcam_list[i].device_name);
		printf(" Product Name		: %s \n",webcam_list[i].product_name);
		printf(" Vendor ID		: 0x%04X \n",webcam_list[i].vendor_id); 
		printf(" Product ID		: 0x%04X \n",webcam_list[i].product_id); 
		printf(" Revision Number	: 0x%04X \n",webcam_list[i].revision_num); 
	}	
end_of_app:
	if(filename)
	{
		free(filename);
		filename=NULL;
	}
	if(v4l_dir)
	{
		closedir(v4l_dir);
		v4l_dir=NULL;
	}
	if(input)
	{
		fclose(input);
		input=NULL;
	}
	if(data_buf)
	{
		free(data_buf);
		data_buf=NULL;
	}
	if(vendor_id_string)
	{
		vendor_id_string=NULL;
	}
	if(product_id_string)
	{
		product_id_string=NULL;
	}
	if(revision_num_string)
	{
		revision_num_string=NULL;
	}
	return 0;
}
